"""Module offering fast graph processing and graph datasets."""

import cpuinfo

required_flags = ("sse", "sse2", "ssse3", "sse4_1", "sse4_2",
                  "avx", "avx2", "bmi1", "bmi2", "popcnt")

flags = cpuinfo.get_cpu_info()["flags"]

unavailable_flags = set(required_flags) - set(flags)

if len(unavailable_flags) > 0:
    raise ValueError(
        (
            "This library was compiled assuming that SIMD instruction "
            "commonly available in CPU hardware since 2013 are present "
            "on the machine where this library is intended to run.\n"
            "On the current machine, the flags {} are not available.\n"
            "You could still compile Ensmallen on this machine and have "
            "a version of the library that can execute here, but the library "
            "has been extensively designed to use SIMD instructions, so "
            "you would have a version slower than the one provided on Pypi."
        ).format(unavailable_flags)
    )

from . import datasets
from .ensmallen import preprocessing  # pylint: disable=import-error
from .ensmallen import Graph  # pylint: disable=import-error
from .ensmallen import edge_list_utils  # pylint: disable=import-error

__all__ = ["edge_list_utils", "Graph", "preprocessing", "datasets"]



"""
This file offers the methods to automatically retrieve the graph PubMedDiabetes.

The graph is automatically retrieved from the LINQS repository. 
The Pubmed Diabetes dataset consists of 19717 scientific publications from
PubMed database pertaining to diabetes classified into one of three classes.
The citation network consists of 44338 links. Each publication in the dataset
is described by a TF/IDF weighted word vector from a dictionary which consists
of 500 unique words.

References
---------------------
Please cite the following if you use the data:

```bib
@inproceedings{namata2012query,
  title={Query-driven active surveying for collective classification},
  author={Namata, Galileo and London, Ben and Getoor, Lise and Huang, Bert and EDU, UMD},
  booktitle={10th International Workshop on Mining and Learning with Graphs},
  volume={8},
  year={2012}
}
```
"""
from typing import Dict
from .parse_linqs import parse_linqs_pubmed_incidence_matrix
from ..automatic_graph_retrieval import AutomaticallyRetrievedGraph
from ...ensmallen import Graph  # pylint: disable=import-error


def PubMedDiabetes(
    directed: bool = False,
    preprocess: bool = True,
    load_nodes: bool = True,
    verbose: int = 2,
    cache: bool = True,
    cache_path: str = "graphs/linqs",
    version: str = "latest",
    **additional_graph_kwargs: Dict
) -> Graph:
    """Return new instance of the PubMedDiabetes graph.

    The graph is automatically retrieved from the LINQS repository.	The Pubmed Diabetes dataset consists of 19717 scientific publications from
	PubMed database pertaining to diabetes classified into one of three classes.
	The citation network consists of 44338 links. Each publication in the dataset
	is described by a TF/IDF weighted word vector from a dictionary which consists
	of 500 unique words.

    Parameters
    -------------------
    directed: bool = False
        Wether to load the graph as directed or undirected.
        By default false.
    preprocess: bool = True
        Whether to preprocess the graph to be loaded in 
        optimal time and memory.
    load_nodes: bool = True,
        Whether to load the nodes vocabulary or treat the nodes
        simply as a numeric range.
    verbose: int = 2,
        Wether to show loading bars during the retrieval and building
        of the graph.
    cache: bool = True
        Whether to use cache, i.e. download files only once
        and preprocess them only once.
    cache_path: str = "graphs"
        Where to store the downloaded graphs.
    version: str = "latest"
        The version of the graph to retrieve.	
    additional_graph_kwargs: Dict
        Additional graph kwargs.

    Returns
    -----------------------
    Instace of PubMedDiabetes graph.

	References
	---------------------
	Please cite the following if you use the data:
	
	```bib
	@inproceedings{namata2012query,
	  title={Query-driven active surveying for collective classification},
	  author={Namata, Galileo and London, Ben and Getoor, Lise and Huang, Bert and EDU, UMD},
	  booktitle={10th International Workshop on Mining and Learning with Graphs},
	  volume={8},
	  year={2012}
	}
	```
    """
    return AutomaticallyRetrievedGraph(
        graph_name="PubMedDiabetes",
        repository="linqs",
        version=version,
        directed=directed,
        preprocess=preprocess,
        load_nodes=load_nodes,
        verbose=verbose,
        cache=cache,
        cache_path=cache_path,
        additional_graph_kwargs=additional_graph_kwargs,
		callbacks=[
			parse_linqs_pubmed_incidence_matrix
		],
		callbacks_arguments=[
		    {
		        "cites_path": "Pubmed-Diabetes/Pubmed-Diabetes/data/Pubmed-Diabetes.DIRECTED.cites.tab",
		        "content_path": "Pubmed-Diabetes/Pubmed-Diabetes/data/Pubmed-Diabetes.NODE.paper.tab",
		        "node_path": "nodes.tsv",
		        "edge_path": "edges.tsv"
		    }
		]
    )()



"""
This file offers the methods to automatically retrieve the graph BSPO.

The graph is automatically retrieved from the KGOBO repository. 


References
---------------------
Please cite the following if you use the data:

```bib
@misc{kgobo,
  title        = "KG-OBO",
  year         = "2021",
  author       = "{Reese, Justin and Caufield, Harry}",
  howpublished = {\\url{https://github.com/Knowledge-Graph-Hub/kg-obo}},
  note = {Online; accessed 14 September 2021}
}
```
"""
from typing import Dict

from ..automatic_graph_retrieval import AutomaticallyRetrievedGraph
from ...ensmallen import Graph  # pylint: disable=import-error


def BSPO(
    directed: bool = False,
    preprocess: bool = True,
    load_nodes: bool = True,
    verbose: int = 2,
    cache: bool = True,
    cache_path: str = "graphs/kgobo",
    version: str = "2021-09-22",
    **additional_graph_kwargs: Dict
) -> Graph:
    """Return new instance of the BSPO graph.

    The graph is automatically retrieved from the KGOBO repository.	

    Parameters
    -------------------
    directed: bool = False
        Wether to load the graph as directed or undirected.
        By default false.
    preprocess: bool = True
        Whether to preprocess the graph to be loaded in 
        optimal time and memory.
    load_nodes: bool = True,
        Whether to load the nodes vocabulary or treat the nodes
        simply as a numeric range.
    verbose: int = 2,
        Wether to show loading bars during the retrieval and building
        of the graph.
    cache: bool = True
        Whether to use cache, i.e. download files only once
        and preprocess them only once.
    cache_path: str = "graphs"
        Where to store the downloaded graphs.
    version: str = "2021-09-22"
        The version of the graph to retrieve.		
	The available versions are:
			- 2021-09-22
    additional_graph_kwargs: Dict
        Additional graph kwargs.

    Returns
    -----------------------
    Instace of BSPO graph.

	References
	---------------------
	Please cite the following if you use the data:
	
	```bib
	@misc{kgobo,
	  title        = "KG-OBO",
	  year         = "2021",
	  author       = "{Reese, Justin and Caufield, Harry}",
	  howpublished = {\\url{https://github.com/Knowledge-Graph-Hub/kg-obo}},
	  note = {Online; accessed 14 September 2021}
	}
	```
    """
    return AutomaticallyRetrievedGraph(
        graph_name="BSPO",
        repository="kgobo",
        version=version,
        directed=directed,
        preprocess=preprocess,
        load_nodes=load_nodes,
        verbose=verbose,
        cache=cache,
        cache_path=cache_path,
        additional_graph_kwargs=additional_graph_kwargs
    )()



"""This sub-module offers methods to automatically retrieve the graphs from KGOBO repository."""

from .mod import MOD
from .fbbt import FBBT
from .bto import BTO
from .chmo import CHMO
from .mco import MCO
from .opmi import OPMI
from .fbdv import FBDV
from .mpath import MPATH
from .spd import SPD
from .vt import VT
from .ehdaa2 import EHDAA2
from .wbls import WBLS
from .rxno import RXNO
from .omp import OMP
from .ero import ERO
from .gno import GNO
from .xco import XCO
from .amphx import AMPHX
from .clyh import CLYH
from .ncro import NCRO
from .iao import IAO
from .exo import EXO
from .swo import SWO
from .obcs import OBCS
from .symp import SYMP
from .taxrank import TAXRANK
from .apo import APO
from .clo import CLO
from .cmo import CMO
from .hso import HSO
from .obi import OBI
from .cheminf import CHEMINF
from .mp import MP
from .labo import LABO
from .olatdv import OLATDV
from .chebi import CHEBI
from .aeo import AEO
from .geno import GENO
from .sbo import SBO
from .uo import UO
from .mop import MOP
from .ogms import OGMS
from .ncbitaxon import NCBITAXON
from .pw import PW
from .fovt import FOVT
from .xpo import XPO
from .zfs import ZFS
from .rs import RS
from .omo import OMO
from .fix import FIX
from .vto import VTO
from .uberon import UBERON
from .bfo import BFO
from .htn import HTN
from .aism import AISM
from .wbbt import WBBT
from .hao import HAO
from .so import SO
from .mondo import MONDO
from .ddpheno import DDPHENO
from .idomal import IDOMAL
from .fbcv import FBCV
from .trans import TRANS
from .scdo import SCDO
from .dron import DRON
from .rbo import RBO
from .ncit import NCIT
from .fma import FMA
from .rex import REX
from .cob import COB
from .sibo import SIBO
from .pdro import PDRO
from .ogg import OGG
from .hancestro import HANCESTRO
from .go import GO
from .plana import PLANA
from .oae import OAE
from .mmusdv import MMUSDV
from .hsapdv import HSAPDV
from .miro import MIRO
from .emapa import EMAPA
from .gecko import GECKO
from .tads import TADS
from .fao import FAO
from .ecao import ECAO
from .opl import OPL
from .tgma import TGMA
from .bco import BCO
from .ico import ICO
from .zeco import ZECO
from .pdumdv import PDUMDV
from .aro import ARO
from .oarcs import OARCS
from .planp import PLANP
from .doid import DOID
from .omrse import OMRSE
from .ppo import PPO
from .ovae import OVAE
from .zp import ZP
from .stato import STATO
from .ecto import ECTO
from .xao import XAO
from .miapa import MIAPA
from .mi import MI
from .ecocore import ECOCORE
from .mmo import MMO
from .eupath import EUPATH
from .obib import OBIB
from .ido import IDO
from .tto import TTO
from .pr import PR
from .nbo import NBO
from .wbphenotype import WBPHENOTYPE
from .cio import CIO
from .clao import CLAO
from .upa import UPA
from .zfa import ZFA
from .ma import MA
from .cdno import CDNO
from .ohd import OHD
from .agro import AGRO
from .txpo import TXPO
from .pato import PATO
from .hom import HOM
from .eco import ECO
from .ddanat import DDANAT
from .bspo import BSPO
from .mro import MRO
from .pco import PCO
from .ornaseq import ORNASEQ
from .hp import HP
from .dpo import DPO
from .cl import CL

__all__ = [
	"MOD", "FBBT", "BTO", "CHMO", "MCO", "OPMI", "FBDV", "MPATH", "SPD", "VT",
	"EHDAA2", "WBLS", "RXNO", "OMP", "ERO", "GNO", "XCO", "AMPHX", "CLYH",
	"NCRO", "IAO", "EXO", "SWO", "OBCS", "SYMP", "TAXRANK", "APO", "CLO", "CMO",
	"HSO", "OBI", "CHEMINF", "MP", "LABO", "OLATDV", "CHEBI", "AEO", "GENO",
	"SBO", "UO", "MOP", "OGMS", "NCBITAXON", "PW", "FOVT", "XPO", "ZFS", "RS",
	"OMO", "FIX", "VTO", "UBERON", "BFO", "HTN", "AISM", "WBBT", "HAO", "SO",
	"MONDO", "DDPHENO", "IDOMAL", "FBCV", "TRANS", "SCDO", "DRON", "RBO", "NCIT",
	"FMA", "REX", "COB", "SIBO", "PDRO", "OGG", "HANCESTRO", "GO", "PLANA",
	"OAE", "MMUSDV", "HSAPDV", "MIRO", "EMAPA", "GECKO", "TADS", "FAO", "ECAO",
	"OPL", "TGMA", "BCO", "ICO", "ZECO", "PDUMDV", "ARO", "OARCS", "PLANP",
	"DOID", "OMRSE", "PPO", "OVAE", "ZP", "STATO", "ECTO", "XAO", "MIAPA",
	"MI", "ECOCORE", "MMO", "EUPATH", "OBIB", "IDO", "TTO", "PR", "NBO", "WBPHENOTYPE",
	"CIO", "CLAO", "UPA", "ZFA", "MA", "CDNO", "OHD", "AGRO", "TXPO", "PATO",
	"HOM", "ECO", "DDANAT", "BSPO", "MRO", "PCO", "ORNASEQ", "HP", "DPO", "CL",
]


"""Module with datasets."""
from . import (kghub, linqs, monarchinitiative, networkrepository,
               pheknowlatorkg, string, yue, zenodo)
from .get_dataset import (get_all_available_graphs_dataframe,
                          get_available_graphs_from_repository,
                          get_available_repositories,
                          get_available_versions_from_graph_and_repository,
                          validate_graph_version,
                          get_dataset)

__all__ = [
    "get_dataset",
    "get_available_repositories",
    "get_available_graphs_from_repository",
    "get_all_available_graphs_dataframe",
    "get_available_versions_from_graph_and_repository",
    "validate_graph_version",
    "kghub", "linqs", "monarchinitiative",
    "networkrepository", "string", "yue", "zenodo", "pheknowlatorkg"
]



"""This sub-module offers methods to automatically retrieve the graphs from KGHub repository."""

from .kgmicrobe import KGMicrobe
from .kgcovid19 import KGCOVID19

__all__ = [
	"KGMicrobe", "KGCOVID19",
]